CREATE DATABASE Kursevi
go

USE Kursevi

--DROP TABLE Student

CREATE TABLE Student(
	Id INT NOT NULL IDENTITY,
	Indeks CHAR(9) NOT NULL,
	Ime VARCHAR(30) NOT NULL,
	Prezime VARCHAR(30) NOT NULL,
	Prosek DECIMAL(5,2) NULL
)

ALTER TABLE Student
ADD CONSTRAINT PK_Student
PRIMARY KEY (Id)

ALTER TABLE Student
ADD CONSTRAINT UQ_Indeks
UNIQUE (Indeks)

ALTER TABLE Student
ADD CONSTRAINT CH_Prosek
CHECK (Prosek >= 5 AND Prosek <=10)

CREATE TABLE Kurs(
	Id INT NOT NULL IDENTITY(1,1) PRIMARY KEY,
	Naziv VARCHAR(500) NOT NULL,
	Sifra VARCHAR(10) NOT NULL UNIQUE,
	ESPB INT NOT NULL CHECK(ESPB>=0 AND ESPB<=30)
)

ALTER TABLE Kurs
ADD CONSTRAINT DF_Kurs_ESPB
DEFAULT 5 FOR ESPB

--DROP TABLE Kurs

CREATE TABLE Predavac(
	Id INT NOT NULL IDENTITY,
	Ime VARCHAR(30) NOT NULL,
	Prezime VARCHAR(30) NOT NULL,
	MaticniBr CHAR(13) NOT NULL
)

ALTER TABLE Predavac
ADD CONSTRAINT PK_Predavac
PRIMARY KEY (Id)

ALTER TABLE Predavac
ADD CONSTRAINT UQ_MaticniBr
UNIQUE (MaticniBr)

ALTER TABLE Predavac
ADD CONSTRAINT CH_MaticniBr
CHECK (ISNUMERIC(MaticniBr) = 1 AND LEN(MaticniBr) = 13)

--DROP TABLE Grupa

CREATE TABLE Grupa(
	Id INT NOT NULL IDENTITY,
	IdK INT NOT NULL,
	IdP INT NULL,
	Oznaka CHAR(2) NOT NULL
)

ALTER TABLE Grupa
ADD CONSTRAINT PK_Grupa
PRIMARY KEY (Id)

ALTER TABLE Grupa
ADD CONSTRAINT FK_Grupa_IdK
FOREIGN KEY (IdK) REFERENCES Kurs(Id)
ON UPDATE CASCADE
ON DELETE CASCADE

ALTER TABLE Grupa
ADD CONSTRAINT FK_Grupa_IdP
FOREIGN KEY (IdP) REFERENCES Predavac(Id)
ON UPDATE CASCADE
ON DELETE SET NULL

/*
ALTER TABLE Grupa
DROP CONSTRAINT FK_Grupa_IdP
*/

CREATE TABLE Pohadja(
	IdS INT NOT NULL FOREIGN KEY REFERENCES Student(Id),
	IdG INT NOT NULL FOREIGN KEY REFERENCES Grupa(Id)
)

ALTER TABLE Pohadja
ADD CONSTRAINT PK_Pohadja
PRIMARY KEY (IdS, IdG)

--DROP TABLE Pohadja

CREATE TABLE IspitniRok(
	Id INT NOT NULL PRIMARY KEY IDENTITY,
	Naziv VARCHAR(500) NOT NULL,
	DatumPocetka DATE NOT NULL,
	DatumKraja DATE NOT NULL
)

ALTER TABLE IspitniRok
ADD CONSTRAINT CH_Datum
CHECK (DatumPocetka <= DatumKraja)

CREATE TABLE Sala(
	Id INT NOT NULL IDENTITY PRIMARY KEY,
	Oznaka VARCHAR(3) NOT NULL UNIQUE
)

CREATE TABLE Ispit(
	Id INT NOT NULL IDENTITY PRIMARY KEY,
	Naziv VARCHAR(500) NOT NULL,
	DatumOdrzavanja DATE NOT NULL,
	Termin INT NOT NULL,
	IdK INT NOT NULL,
	IdIR INT NOT NULL,
	IdS INT NOT NULL
)

ALTER TABLE Ispit
ADD CONSTRAINT FK_Ispit_IdK
FOREIGN KEY (IdK) REFERENCES Kurs(Id)
ON UPDATE CASCADE
ON DELETE NO ACTION

ALTER TABLE Ispit
ADD CONSTRAINT FK_Ispit_IdIR
FOREIGN KEY (IdIR) REFERENCES IspitniRok(Id)
ON UPDATE CASCADE
ON DELETE NO ACTION

ALTER TABLE Ispit
ADD CONSTRAINT FK_Ispit_IdS
FOREIGN KEY (IdS) REFERENCES Sala(Id)
ON UPDATE CASCADE
ON DELETE NO ACTION

ALTER TABLE Ispit
ADD CONSTRAINT UQ_Termin
UNIQUE (DatumOdrzavanja, Termin, IdS)

/*
alter table Ispit
drop constraint FK_Ispit_IdS
*/

CREATE TABLE Ocena(
	Id INT NOT NULL IDENTITY PRIMARY KEY,
	Ocena INT NOT NULL CHECK (Ocena >=3 AND Ocena<=10),
	IdS INT NOT NULL,
	IdI INT NOT NULL
)

ALTER TABLE Ocena
ADD CONSTRAINT FK_Ocena_IdS
FOREIGN KEY (IdS) REFERENCES Student(Id)
ON UPDATE CASCADE
ON DELETE NO ACTION

ALTER TABLE Ocena
ADD CONSTRAINT FK_Ocena_IdI
FOREIGN KEY (IdI) REFERENCES Ispit(Id)
ON UPDATE CASCADE
ON DELETE NO ACTION