
CREATE FUNCTION fRacunajProsek
(
    @idS int
)
RETURNS DECIMAL(5,2)
AS
BEGIN
    RETURN (
		select avg(cast(O.Ocena as decimal(5,2)))
		from Ispit I, Ocena O
		where O.IdI = I.Id and O.IdS = @idS
		and I.DatumOdrzavanja = (
			select MAX(I2.DatumOdrzavanja)
			from Ispit I2, Ocena O2
			where I2.IdK = I.IdK and O2.IdI = I2.Id and O2.IdS = @idS
		)
		and O.Ocena > 5
	)

END
go

CREATE PROCEDURE spIzmeniProsek
    @idS int
AS
    update Student
	set Prosek = dbo.fRacunajProsek(@idS)
	where id = @idS
RETURN 0 
go

CREATE TRIGGER racunajProsek
    ON Ocena
    FOR DELETE, INSERT, UPDATE
    AS
    BEGIN
		declare @brI int
		declare @brD int
		declare @idSStari int, @idSNovi int
		
		declare @kursorI cursor, @kursorD cursor

		set @kursorI = cursor for
		select IdS
		from inserted

		set @kursorD = cursor for
		select IdS
		from deleted

		select @brI = count(*)
		from inserted

		select  @brD = COUNT(*)
		from deleted

		if(@brI = 0) -- delete
			begin
				open @kursorD

				fetch next from @kursorD
				into @IdSStari

				while @@FETCH_STATUS = 0
				begin
					execute spIzmeniProsek @idSStari

					fetch next from @kursorD
					into @IdSStari
				end

				close @kursorD
				deallocate @kursorD
			end
		else if(@brD = 0) -- insert
			begin
				open @kursorI

				fetch next from @kursorI
				into @IdSNovi

				while @@FETCH_STATUS = 0
				begin
					execute spIzmeniProsek @IdSNovi

					fetch next from @kursorI
					into @IdSNovi
				end

				close @kursorI
				deallocate @kursorI
			end
		else -- update
			begin
				open @kursorI
				open @kursorD

				fetch next from @kursorI
				into @IdSNovi

				fetch next from @kursorD
				into @IdSStari

				while @@FETCH_STATUS = 0
				begin
					execute spIzmeniProsek @idSNovi
					if(@idSStari <> @idSNovi)
						execute spIzmeniProsek @idSStari

					fetch next from @kursorI
					into @IdSNovi

					fetch next from @kursorD
					into @IdSStari
				end

				close @kursorI
				deallocate @kursorI

				close @kursorD
				deallocate @kursorD
			end
    END
go

CREATE TRIGGER racunajProsek2
    ON Ispit
    FOR UPDATE
    AS
    BEGIN
		declare @kursorI cursor
		declare @kursorD cursor

		declare @idI int
		declare @idIRS int, @idKS int
		declare @idIRN int, @idKN int

		declare @studenti TABLE (IdS int)
		
		set @kursorI = cursor for
		select Id, IdIR, IdK
		from inserted

		set @kursorD = cursor for
		select IdIR, IdK
		from deleted

		open @kursorI
		open @kursorD

		fetch from @kursorI
		into @IdI, @IdIRN, @IdKN

		fetch from @kursorD
		into @IdIRS, @IdKS

		while @@FETCH_STATUS = 0
		begin
			if(@IdIRS <> @IdIRN or @idKS <> @IdKN)
			begin
				insert into @studenti
				select O.IdS
				from Ispit I, Ocena O
				where O.IdI = I.Id and I.Id = @idI
			end

			fetch from @kursorI
			into @IdI, @IdIRN, @IdKN

			fetch from @kursorD
			into @IdIRS, @IdKS
		end

		close @kursorI
		deallocate @kursorI

		close @kursorD
		deallocate @kursorD

		declare @kursorStudenti cursor
		declare @idS int

		set @kursorStudenti = cursor for
		select IdS
		from @studenti

		open @kursorStudenti

		fetch from @kursorStudenti
		into @idS

		while @@FETCH_STATUS = 0
		begin
			execute spIzmeniProsek @idS
			
			fetch from @kursorStudenti
			into @idS
		end

		close @kursorStudenti
		deallocate @kursorStudenti
    END
go

create TRIGGER proveraGrupa
    ON Pohadja
    FOR INSERT, UPDATE
    AS
    BEGIN
		if(exists(
			select G.IdK, P.IdS
			from Pohadja P, Grupa G
			where P.IdG = G.Id
			group by G.IdK, P.IdS
			having (count(*) > 1))
		)
		rollback transaction;
    END
go

create TRIGGER proveraGrupa2
    ON Grupa
    FOR INSERT, UPDATE
    AS
    BEGIN
		if(exists(
			select G.IdK, P.IdS
			from Pohadja P, Grupa G
			where P.IdG = G.Id
			group by G.IdK, P.IdS
			having (count(*) > 1))
		)
		rollback transaction;
    END
go
